# -*- coding: utf-8 -*-
################################################################################
#
#  Rattail -- Retail Software Framework
#  Copyright © 2010-2015 Lance Edgar
#
#  This file is part of Rattail.
#
#  Rattail is free software: you can redistribute it and/or modify it under the
#  terms of the GNU Affero General Public License as published by the Free
#  Software Foundation, either version 3 of the License, or (at your option)
#  any later version.
#
#  Rattail is distributed in the hope that it will be useful, but WITHOUT ANY
#  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
#  FOR A PARTICULAR PURPOSE.  See the GNU Affero General Public License for
#  more details.
#
#  You should have received a copy of the GNU Affero General Public License
#  along with Rattail.  If not, see <http://www.gnu.org/licenses/>.
#
################################################################################
"""
Database Stuff
"""

from __future__ import unicode_literals

from sqlalchemy import orm
from sqlalchemy.orm.exc import NoResultFound


class SessionBase(orm.Session):
    """
    Custom SQLAlchemy session class, which adds some convenience methods
    related to the SQLAlchemy-Continuum integration.
    """

    def __init__(self, continuum_user=None, **kwargs):
        """
        Custom constructor, to allow specifying the Continuum user at session
        creation.  If ``continuum_user`` is specified, its value will be passed
        to :meth:`set_continuum_user()`.
        """
        super(SessionBase, self).__init__(**kwargs)
        if continuum_user is None:
            self.continuum_user = None
        else:
            self.set_continuum_user(continuum_user)

    def set_continuum_user(self, user_info):
        """
        Set the effective Continuum user for the session.

        :param user_info: May be a :class:`model.User` instance, or the
          ``uuid`` or ``username`` for one.
        """
        from rattail.db import model

        if isinstance(user_info, model.User):
            user = user_info
        else:
            user = self.query(model.User).get(user_info)
            if not user:
                try:
                    user = self.query(model.User).filter_by(username=user_info).one()
                except NoResultFound:
                    user = None
        self.continuum_user = user


Session = orm.sessionmaker(class_=SessionBase)



# -*- coding: utf-8 -*-
################################################################################
#
#  Rattail -- Retail Software Framework
#  Copyright © 2010-2014 Lance Edgar
#
#  This file is part of Rattail.
#
#  Rattail is free software: you can redistribute it and/or modify it under the
#  terms of the GNU Affero General Public License as published by the Free
#  Software Foundation, either version 3 of the License, or (at your option)
#  any later version.
#
#  Rattail is distributed in the hope that it will be useful, but WITHOUT ANY
#  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
#  FOR A PARTICULAR PURPOSE.  See the GNU Affero General Public License for
#  more details.
#
#  You should have received a copy of the GNU Affero General Public License
#  along with Rattail.  If not, see <http://www.gnu.org/licenses/>.
#
################################################################################
"""
Configuration Utilities
"""

from __future__ import unicode_literals

import sys
import shlex

from edbob.configuration import AppConfigParser


def parse_list(value):
    """
    Parse a configuration value, splitting by whitespace and/or commas and
    taking quoting into account, etc., yielding a list of strings.
    """
    if value is None:
        return []
    # Per the shlex docs (https://docs.python.org/2/library/shlex.html):
    # "Prior to Python 2.7.3, this module did not support Unicode input."
    if sys.version_info < (2, 7, 3) and isinstance(value, unicode): # pragma: no cover
        value = value.encode(u'utf-8')
    parser = shlex.shlex(value)
    parser.whitespace += u','
    parser.whitespace_split = True
    values = list(parser)
    for i, value in enumerate(values):
        if value.startswith(u'"') and value.endswith(u'"'):
            values[i] = value[1:-1]
    return values


class RattailConfig(object):
    """
    Rattail configuration object.  Represents the sum total of configuration in
    effect for a running app.  This is essentially a thin wrapper over the
    standard library's ``ConfigParser`` class(es); in fact any attribute not
    found in this class will fall back to the parser.

    .. warning::
       This API should hardly be considered stable.
    """

    def __init__(self, parser=None):
        """
        Create a config object.  You may optionally specify the config parser
        instance; if none is provided then an instance of
        ``edbob.configuration.AppConfigParser`` will be created.
        """
        if parser is None:
            parser = AppConfigParser('rattail')
        self.parser = parser

    def __getattr__(self, name):
        return getattr(self.parser, name)

    def getint(self, section, option, default=None):
        """
        Overriddes base class method to allow for a default.
        """
        try:
            val = self.parser.getint(section, option)
        except TypeError:
            return default
        return val

    def setdefault(self, section, option, value):
        """
        Sets a default value for given section and option, if no value exists
        there yet.  The final value (whether pre-existing or newly-set) is
        returned.
        """
        exists = True           # start by assuming value exists
        if not self.parser.has_section(section):
            self.parser.add_section(section)
            exists = False
        elif not self.parser.has_option(section, option):
            exists = False
        if not exists:
            self.parser.set(section, option, value)
        return self.parser.get(section, option)



# -*- coding: utf-8 -*-
################################################################################
#
#  Rattail -- Retail Software Framework
#  Copyright © 2010-2015 Lance Edgar
#
#  This file is part of Rattail.
#
#  Rattail is free software: you can redistribute it and/or modify it under the
#  terms of the GNU Affero General Public License as published by the Free
#  Software Foundation, either version 3 of the License, or (at your option)
#  any later version.
#
#  Rattail is distributed in the hope that it will be useful, but WITHOUT ANY
#  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
#  FOR A PARTICULAR PURPOSE.  See the GNU Affero General Public License for
#  more details.
#
#  You should have received a copy of the GNU Affero General Public License
#  along with Rattail.  If not, see <http://www.gnu.org/licenses/>.
#
################################################################################

from __future__ import unicode_literals

import sys
import os.path
from setuptools import setup, find_packages


here = os.path.abspath(os.path.dirname(__file__))
execfile(os.path.join(here, 'rattail', '_version.py'))
README = open(os.path.join(here, 'README.rst')).read()


requires = [
    #
    # Version numbers within comments below have specific meanings.
    # Basically the 'low' value is a "soft low," and 'high' a "soft high."
    # In other words:
    #
    # If either a 'low' or 'high' value exists, the primary point to be
    # made about the value is that it represents the most current (stable)
    # version available for the package (assuming typical public access
    # methods) whenever this project was started and/or documented.
    # Therefore:
    #
    # If a 'low' version is present, you should know that attempts to use
    # versions of the package significantly older than the 'low' version
    # may not yield happy results.  (A "hard" high limit may or may not be
    # indicated by a true version requirement.)
    #
    # Similarly, if a 'high' version is present, and especially if this
    # project has laid dormant for a while, you may need to refactor a bit
    # when attempting to support a more recent version of the package.  (A
    # "hard" low limit should be indicated by a true version requirement
    # when a 'high' version is present.)
    #
    # In any case, developers and other users are encouraged to play
    # outside the lines with regard to these soft limits.  If bugs are
    # encountered then they should be filed as such.
    #
    # package                           # low                   high

    # 'decorator',                        # 3.3.2
    'lockfile',                         # 0.9.1
    'progressbar',                      # 2.3

    # TODO: Remove this / make it optional / etc.
    'Mako',                             # 1.0.0

    # Hardcode ``pytz`` minimum since apparently it isn't (any longer?) enough
    # to simply require the library.
    'pytz>=2013b',                      #                       2013b 

    # This will someday be removed...
    'edbob>=0.1a29',                    #                       0.1a29
    ]


if sys.platform != 'win32':
    requires += [
        #
        # package                       # low                   high
            
        'pyinotify',                    # 0.9.3
        ]


# Python < 2.7 has a standard library in need of supplementation.
if sys.version_info < (2, 7):
    requires += [
        #
        # package                       # low                   high

        'argparse',                     # 1.2.1
        'ordereddict',                  # 1.1
        ]


extras = {

    'bouncer': [
        #
        # package                       # low                   high

        'flufl.bounce',                 # 2.3
    ],

    'db': [
        #
        # package                       # low                   high

        'alembic',                      # 0.6.0
        'SQLAlchemy',                   # 0.7.6
        'SQLAlchemy-Continuum',         # 1.1.5

        # TODO: Looks like SQLAlchemy-Continuum is rather out of date and the API
        # for SQLAlchemy-Utils has broken for it somehow as of 0.30.15.  For now
        # this is easiest but maybe should submit a patch to Continuum project?
        'SQLAlchemy-Utils<0.30.15',     #                       0.30.14
        ],

    'vendors': [
        #
        # package                       # low                   high

        'xlrd',                         # 0.9.3
        ],

    'docs': [
        #
        # package                       # low                   high

        'Sphinx',                       # 1.1.3
        ],

    'tests': [
        #
        # package                       # low                   high

        'coverage',                     # 3.6
        u'fixture',                     # 1.5
        'mock',                         # 1.0.1
        'nose',                         # 1.3.0
        ],
    }


if sys.platform == 'win32':
    extras['auth'] = [
        #
        # package                       # low                   high

        'py-bcrypt-w32',                # 0.2.2
        ]
else:
    extras['auth'] = [
        #
        # package                       # low                   high
            
        'py-bcrypt',                    # 0.2
        ]


setup(
    name = "rattail",
    version = __version__,
    author = "Lance Edgar",
    author_email = "lance@edbob.org",
    url = "http://rattailproject.org/",
    license = "GNU Affero GPL v3",
    description = "Retail Software Framework",
    long_description = README,

    classifiers = [
        'Development Status :: 3 - Alpha',
        'Environment :: Console',
        'Environment :: Web Environment',
        'Environment :: Win32 (MS Windows)',
        'Environment :: X11 Applications',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: GNU Affero General Public License v3',
        'Natural Language :: English',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Topic :: Office/Business',
        'Topic :: Software Development :: Libraries :: Python Modules',
        ],

    install_requires = requires,
    extras_require = extras,
    tests_require = [u'rattail[db,tests]'],
    test_suite = 'nose.collector',

    packages = find_packages(exclude=['tests.*', 'tests']),
    include_package_data = True,
    zip_safe = False,

    entry_points = """

[console_scripts]
rattail = rattail.commands:main

[gui_scripts]
rattailw = rattail.commands:main

[rattail.commands]
adduser = rattail.commands:AddUser
bouncer = rattail.commands:EmailBouncer
date-organize = rattail.commands:DateOrganize
dbsync = rattail.commands:DatabaseSyncCommand
dump = rattail.commands:Dump
filemon = rattail.commands:FileMonitorCommand
import-csv = rattail.commands:ImportCSV
initdb = rattail.commands:InitializeDatabase
load-host-data = rattail.commands:LoadHostDataCommand
make-config = rattail.commands:MakeConfigCommand
make-user = rattail.commands:MakeUserCommand
palm = rattail.commands:PalmCommand
purge-batches = rattail.commands:PurgeBatchesCommand

[rattail.batches.providers]
print_labels = rattail.batches.providers.labels:PrintLabels

[rattail.sil.column_providers]
rattail = rattail.sil.columns:provide_columns

[rattail.vendors.catalogs.parsers]
rattail.contrib.dutchvalley = rattail.contrib.vendors.catalogs.dutchvalley:DutchValleyCatalogParser
rattail.contrib.kehe = rattail.contrib.vendors.catalogs.kehe:KeheCatalogParser
rattail.contrib.lotuslight = rattail.contrib.vendors.catalogs.lotuslight:LotusLightCatalogParser
rattail.contrib.unfi = rattail.contrib.vendors.catalogs.unfi:UNFICatalogParser

[rattail.vendors.invoices.parsers]
rattail.contrib.alberts = rattail.contrib.vendors.invoices.alberts:AlbertsInvoiceParser
rattail.contrib.kehe = rattail.contrib.vendors.invoices.kehe:KeheInvoiceParser
rattail.contrib.unfi = rattail.contrib.vendors.invoices.unfi:UnfiInvoiceParser

""",
    )
